// Matrix ARM bootloader V1.0c
// David Miles 2006
// Flash.h - Flash handling code, based on Atmel flash example


#ifndef Flash_h
#define Flash_h

/*-------------------------------*/
/* Flash Status Field Definition */
/*-------------------------------*/

#define 	AT91C_MC_FSR_MVM 		((unsigned int) 0xFF << 8)		// (MC) Status Register GPNVMx: General-purpose NVM Bit Status
#define 	AT91C_MC_FSR_LOCK 		((unsigned int) 0xFFFF << 16)	// (MC) Status Register LOCKSx: Lock Region x Lock Status


#define	 ERASE_VALUE 		0xFFFFFFFF

/*-----------------------*/
/* Flash size Definition */
/*-----------------------*/
/* 64 Kbytes of Internal High-speed Flash, Organized in 512 Pages of 256 Bytes */

//#define  FLASH_PAGE_SIZE_BYTE	256
//#define  FLASH_PAGE_SIZE_LONG	64

//#define  FLASH_LOCK_BITS_SECTOR	16
#define  FLASH_SECTOR_PAGE		64
//#define  FLASH_LOCK_BITS		8    /* 16 lock bits, each protecting 16 sectors of 32 pages*/
#define  FLASH_BASE_ADDRESS		0x00100000
/*------------------------------*/
/* External function Definition */
/*------------------------------*/

/* Flash function */
RAMFUNC extern void AT91F_Flash_Init(void);
extern int AT91F_Flash_Check_Erase(unsigned int * start, unsigned int size);
RAMFUNC extern int AT91F_Flash_Erase_All(void);
RAMFUNC extern int AT91F_Flash_Write( unsigned int Flash_Address ,int size ,unsigned int * buff);
RAMFUNC extern int AT91F_Flash_Write2( unsigned int Flash_Address ,int size ,unsigned int * buff);
RAMFUNC extern int AT91F_Flash_Write_all( unsigned int Flash_Address ,int size ,unsigned int * buff);

/* Lock Bits functions */
RAMFUNC extern int AT91F_Flash_Lock_Status(void);
RAMFUNC extern int AT91F_Flash_Lock (unsigned int Flash_Lock);
RAMFUNC extern int AT91F_Flash_Unlock(unsigned int Flash_Lock);

/* NVM bits functions */
RAMFUNC extern int AT91F_NVM_Status(void);
RAMFUNC extern int AT91F_NVM_Set (unsigned char NVM_Number);
RAMFUNC extern int AT91F_NVM_Clear(unsigned char NVM_Number);

/* Security bit function */
RAMFUNC extern int AT91F_SET_Security_Status (void);
RAMFUNC extern int AT91F_SET_Security (void);

RAMFUNC extern unsigned int FLASH_PAGE_SIZE_BYTE;

#endif /* Flash_h */
