/*----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*----------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*----------------------------------------------------------------------------
* File Name           : Board.h
* Object              : AT91SAM7S Evaluation Board Features Definition File.
*
* Creation            : JPP   16/Jun/2004
* V 1.0 21/Feb/05 JPP : Define __ramfunc
* V 1.1 21/Feb/05 JPP : add Lib definition
* V 1.2 22/Feb/05 JPP : Add DBGU inline definition
*----------------------------------------------------------------------------
*/
#ifndef Board_h
#define Board_h

#include "AT91SAM7S128.h"
#define __inline extern inline
#include "lib_AT91SAM7S128.h"
//* inline definition

#define __ramfunc

#define AT91C_US_ASYNC_MODE ( AT91C_US_USMODE_NORMAL + AT91C_US_NBSTOP_1_BIT + AT91C_US_PAR_NONE + AT91C_US_CHRL_8_BITS + AT91C_US_CLKS_CLOCK )

#define true	-1
#define false	0

/*-------------------------------*/
/* SAM7Board Memories Definition */
/*-------------------------------*/
// The AT91SAM128 embeds a 32-Kbyte SRAM bank, and 128 K-Byte Flash

#define  INT_SARM           0x00200000
#define  INT_SARM_REMAP	    0x00000000

#define  INT_FLASH          0x00000000
#define  INT_FLASH_REMAP    0x01000000

#define  FLASH_PAGE_NB	    512
#define  FLASH_PAGE_LOCK    32
#define  FLASH_PAGE_SIZE    128

/*-----------------*/
/* Ports definition*/
/*-----------------*/

#define PORTA0 			( 1<<0 )
#define PORTA1			( 1<<1 )
#define PORTA2			( 1<<2 )
#define PORTA3			( 1<<3 )
#define PORTA4			( 1<<4 )
#define PORTA5			( 1<<5 )
#define PORTA6			( 1<<6 )
#define PORTA7			( 1<<7 )

#define PORTA (PORTA0|PORTA1|PORTA2|PORTA3|PORTA4|PORTA5|PORTA6|PORTA7)

#define PORTB0			( 1<<8 )
#define PORTB1			( 1<<9 )
#define PORTB2			( 1<<10 )
#define PORTB3			( 1<<11 )
#define PORTB4			( 1<<12 )
#define PORTB5			( 1<<13 )
#define PORTB6			( 1<<14 )
#define PORTB7			( 1<<15 )

#define PORTB (PORTB0|PORTB1|PORTB2|PORTB3|PORTB4|PORTB5|PORTB6|PORTB7)

#define PORTC0			( 1<<24 ) /* RTS1 */
#define PORTC1			( 1<<26 )
#define PORTC2			( 1<<27 )
#define PORTC3			( 1<<28 )
#define PORTC4			( 1<<25 ) /* CTS1 */
#define PORTC5			( 1<<23 )
#define PORTC6			( 1<<22 ) /* TXD1 */
#define PORTC7			( 1<<21 ) /* RXD1 */

#define PORTC (PORTC0|PORTC1|PORTC2|PORTC3|PORTC4|PORTC5|PORTC6|PORTC7)

#define PORTD0			( 1<<17 ) /* AD0 */
#define PORTD1			( 1<<18 ) /* AD1 */
#define PORTD2			( 1<<19 ) /* AD2 */
#define PORTD3			( 1<<20 ) /* AD3 */
#define PORTD4			( 1<<16 )
#define PORTD5			( 1<<29 )
#define PORTD6			( 1<<30 )
#define PORTD7			( 1<<31 )

#define PORTD (PORTD0|PORTD1|PORTD2|PORTD3|PORTD4|PORTD5|PORTD6|PORTD7)

/*--------------*/
/* Master Clock */
/*--------------*/

#define EXT_OC          18432000   // Exetrnal ocilator MAINCK
#define MCK             47923200   // MCK (PLLRC div by 2)
#define MCKKHz          (MCK/1000) //


/* 
   RAMFUNC 
   Attribute which defines a function to be located 
   in memory section .fastrun and called via "long calls". 
   See linker-skript and startup-code to see how the
   .fastrun-section is handled.
   The definition is not only useful for ISRs but since
   ISRs should be executed fast the macro is defined in
   this header
*/
#define RAMFUNC __attribute__ ((long_call, section (".ramfunc")))

// #define LCD

#endif /* Board_h */
